@@ -8,6 +8,7 @@ _SystemDiagrams = vss_utils._SystemDiagrams
 _Graphs = graph_utils._Graphs
 _DataFiles = misc_utils._DataFiles
 _Optimization = misc_utils._Optimization
+_Yield = misc_utils._Yield
 _GlobalDefinitions = misc_utils._GlobalDefinitions
 _CommonTools = common_utils._CommonTools
 #
@@ -74,7 +75,7 @@ def establish_link(version_str: str ='', clsid_str:str ='') -> object:
     return awrde
     #
 class Project(_Graphs, _Schematics, _SystemDiagrams, _DataFiles, _Optimization, _GlobalDefinitions,
-              _CommonTools):
+              _CommonTools, _Yield):
     ''' 
     Highest level class in pyawr_utils. User interface to all classes,methods, properties,
     etc. are through the Project class
@@ -92,6 +93,7 @@ class Project(_Graphs, _Schematics, _SystemDiagrams, _DataFiles, _Optimization,
             _SystemDiagrams.__init__(self, awrde)
             _DataFiles.__init__(self, awrde)
             _Optimization.__init__(self, awrde)
+            _Yield.__init__(self, awrde)
             _GlobalDefinitions.__init__(self, awrde)            
         #end if
         _CommonTools.__init__(self, awrde)
