@@ -19,7 +19,7 @@ class _CommonTools():
             #end for
         #end for
         return active_meas_list
-        #    
+        #
 #
 #*****************************************************************************
 #
@@ -217,7 +217,124 @@ class _Parameter():
             raise TypeError('step_size_value must be float or int')
         #end if
         self._parameter.StepSize = step_size_value
+        #
+    @property
+    def use_statistics(self) -> bool:#------------------------------------------
+        '''returns the UseStatistics setting'''
+        return self._parameter.UseStatistics
+        #
+    @use_statistics.setter
+    def use_statistics(self, enabled:bool):#--------------------------------------
+        '''sets the UseStatistics parameter'''
+        self._parameter.UseStatistics = enabled
+        #
+    @property
+    def yield_optimize(self) -> bool:#-----------------------------------------
+        '''returns Yield Optimize setting'''
+        return self._parameter.OptimizeYield
+        #
+    @yield_optimize.setter
+    def yield_optimize(self, enabled:bool):#-------------------------------------
+        '''sets the OptimizeYield parameter'''
+        self._parameter.OptimizeYield = enabled
+        #
+    @property 
+    def yield_distribution(self) -> str:#------------------------------------------
+        '''returns name of the Yield Distribution'''
+        distribution_str = self._get_yield_distribution(self._parameter.Distribution)
+        return distribution_str
+        #
+    @yield_distribution.setter
+    def yield_distribution(self, distribution_name:str):#---------------------------
+        '''
+        sets the yield distribution
+        
+        Parameters
+        ----------
+        
+        distribution_name:string
+                    Valid strings: Uniform, Normal, Log-Normal, Discrete, Normal Minus Tol,
+                                   Normal Clipped
+        '''
+        distribution_val = self._set_yield_distribution(distribution_name)
+        if distribution_val == -999:
+            raise RuntimeError('Invalid distribution_name: '+distribution_name)
+        else:
+            self._parameter.Distribution = distribution_val
+        #end if
+        #
+    @property
+    def yield_tol_in_percent(self) -> bool:#-------------------------------------------
+        '''returns the Tolerance in Percent setting'''
+        return self._parameter.TolInPercent
+        #
+    @yield_tol_in_percent.setter
+    def yield_tol_in_percent(self, enabled:bool):#-----------------------------------
+        ''' sets Tolerance In Percent'''
+        self._parameter.TolInPercent = enabled
+        #
+    @property
+    def yield_statistical_variation(self) -> float:#-----------------------------------
+        '''returns the statistical variation value'''
+        return self._parameter.StatVariation
+        #
+    @yield_statistical_variation.setter
+    def yield_statistical_variation(self, statistical_variation_val:float):#-----------------------------------
+        '''sets the statistical variation value'''
+        if (not isinstance(statistical_variation_val,float)) and (not isinstance(statistical_variation_val,int)):
+            raise TypeError('statistical_variation_val must be float type')
+        #end if
+        self._parameter.StatVariation = statistical_variation_val
+        #
+    @property
+    def yield_statistical_variation_2(self) -> float:#-----------------------------------
+        '''returns the statistical variation 2 value'''
+        return self._parameter.StatVariation2
+        #
+    @yield_statistical_variation_2.setter
+    def yield_statistical_variation_2(self, statistical_variation_val:float):#-----------------------------------
+        '''sets the statistical variation value'''
+        if (not isinstance(statistical_variation_val,float)) and (not isinstance(statistical_variation_val,int)):
+            raise TypeError('statistical_variation_val must be float type')
+        #end if
+        self._parameter.StatVariation2 = statistical_variation_val
         #    
+    def _get_yield_distribution(self, param_distribution):#-------------------------
+        if param_distribution == 1:
+            distribution_str = 'Uniform'
+        elif param_distribution == 2:
+            distribution_str = 'Normal'
+        elif param_distribution == 4:
+            distribution_str = 'Log-Normal'
+        elif param_distribution == 5:
+            distribution_str = 'Discrete'
+        elif param_distribution == 6:
+            distribution_str = 'Normal Minus Tol'
+        elif param_distribution == 7:
+            distribution_str = 'Normal Clipped' 
+        else:
+            distribution_str = 'Not Recognized'
+        #end if
+        return distribution_str
+        #
+    def _set_yield_distribution(self, distribution_str):#-------------------------
+        if  distribution_str == 'Uniform':
+            distribution_val = 1
+        elif distribution_str == 'Normal':
+            distribution_val = 2
+        elif  distribution_str == 'Log-Normal':
+            distribution_val = 4
+        elif distribution_str == 'Discrete':
+            distribution_val = 5
+        elif  distribution_str == 'Normal Minus Tol':
+            distribution_val = 6
+        elif distribution_str == 'Normal Clipped':
+            distribution_val = 7
+        else:
+            distribution_val = -999
+        #end if
+        return distribution_val
+        #
 class _Parameters():
     def __init__(self, element, document_type):#----------------------------------------------
         self._element = element
@@ -779,6 +896,87 @@ class _Equation():
             raise TypeError('step_size_value must be float or int')
         #end if
         self._eqn_doc.Equations(self._eqn_index).StepSize = step_size_value
+        #
+    @property
+    def use_statistics(self) -> bool:#------------------------------------------
+        '''returns the UseStatistics setting'''
+        return self._eqn_doc.Equations(self._eqn_index).UseStatistics
+        #
+    @use_statistics.setter
+    def use_statistics(self, enabled:bool):#--------------------------------------
+        '''sets the UseStatistics parameter'''
+        self._eqn_doc.Equations(self._eqn_index).UseStatistics = enabled
+        #
+    @property
+    def yield_optimize(self) -> bool:#-----------------------------------------
+        '''returns Yield Optimize setting'''
+        return self._eqn_doc.Equations(self._eqn_index).OptimizeYield
+        #
+    @yield_optimize.setter
+    def yield_optimize(self, enabled:bool):#-------------------------------------
+        '''sets the OptimizeYield parameter'''
+        self._eqn_doc.Equations(self._eqn_index).OptimizeYield = enabled
+        #
+    @property 
+    def yield_distribution(self) -> str:#------------------------------------------
+        '''returns name of the Yield Distribution'''
+        distribution_str = self._get_yield_distribution(self._eqn_doc.Equations(self._eqn_index).Distribution)
+        return distribution_str
+        #
+    @yield_distribution.setter
+    def yield_distribution(self, distribution_name:str):#---------------------------
+        '''
+        sets the yield distribution
+        
+        Parameters
+        ----------
+        
+        distribution_name:string
+                    Valid strings: Uniform, Normal, Log-Normal, Discrete, Normal Minus Tol,
+                                   Normal Clipped
+        '''
+        distribution_val = self._set_yield_distribution(distribution_name)
+        if distribution_val == -999:
+            raise RuntimeError('Invalid distribution_name: '+distribution_name)
+        else:
+            self._eqn_doc.Equations(self._eqn_index).Distribution = distribution_val
+        #end if
+        #
+    @property
+    def yield_tol_in_percent(self) -> bool:#-------------------------------------------
+        '''returns the Tolerance in Percent setting'''
+        return self._eqn_doc.Equations(self._eqn_index).TolInPercent
+        #
+    @yield_tol_in_percent.setter
+    def yield_tol_in_percent(self, enabled:bool):#-----------------------------------
+        ''' sets Tolerance In Percent'''
+        self._eqn_doc.Equations(self._eqn_index).TolInPercent = enabled
+        #
+    @property
+    def yield_statistical_variation(self) -> float:#-----------------------------------
+        '''returns the statistical variation value'''
+        return self._eqn_doc.Equations(self._eqn_index).StatVariation
+        #
+    @yield_statistical_variation.setter
+    def yield_statistical_variation(self, statistical_variation_val:float):#-----------------------------------
+        '''sets the statistical variation value'''
+        if (not isinstance(statistical_variation_val,float)) and (not isinstance(statistical_variation_val,int)):
+            raise TypeError('statistical_variation_val must be float type')
+        #end if
+        self._eqn_doc.Equations(self._eqn_index).StatVariation = statistical_variation_val
+        #
+    @property
+    def yield_statistical_variation_2(self) -> float:#-----------------------------------
+        '''returns the statistical variation 2 value'''
+        return self._eqn_doc.Equations(self._eqn_index).StatVariation2
+        #
+    @yield_statistical_variation_2.setter
+    def yield_statistical_variation_2(self, statistical_variation_val:float):#-----------------------------------
+        '''sets the statistical variation value'''
+        if (not isinstance(statistical_variation_val,float)) and (not isinstance(statistical_variation_val,int)):
+            raise TypeError('statistical_variation_val must be float type')
+        #end if
+        self._eqn_doc.Equations(self._eqn_index).StatVariation2 = statistical_variation_val
         #        
     def _update_equation(self):#---------------------------------------------------------
         '''write equation to AWR document'''
@@ -848,7 +1046,43 @@ class _Equation():
             #end if
         #end if
         return variable_type_str
-        #  
+        #
+    def _get_yield_distribution(self, param_distribution):#-------------------------
+        if param_distribution == 1:
+            distribution_str = 'Uniform'
+        elif param_distribution == 2:
+            distribution_str = 'Normal'
+        elif param_distribution == 4:
+            distribution_str = 'Log-Normal'
+        elif param_distribution == 5:
+            distribution_str = 'Discrete'
+        elif param_distribution == 6:
+            distribution_str = 'Normal Minus Tol'
+        elif param_distribution == 7:
+            distribution_str = 'Normal Clipped' 
+        else:
+            distribution_str = 'Not Recognized'
+        #end if
+        return distribution_str
+        #
+    def _set_yield_distribution(self, distribution_str):#-------------------------
+        if  distribution_str == 'Uniform':
+            distribution_val = 1
+        elif distribution_str == 'Normal':
+            distribution_val = 2
+        elif  distribution_str == 'Log-Normal':
+            distribution_val = 4
+        elif distribution_str == 'Discrete':
+            distribution_val = 5
+        elif  distribution_str == 'Normal Minus Tol':
+            distribution_val = 6
+        elif distribution_str == 'Normal Clipped':
+            distribution_val = 7
+        else:
+            distribution_val = -999
+        #end if
+        return distribution_val
+        #    
 #
 #**********************************************************************************************
 #    
