@@ -1139,7 +1139,7 @@ class _Optimization():
         TotalCost_ay = np.array(TotalCost_list)
         return TotalCost_ay
         #
-    def _build_opt_goal_dict(self) -> dict:#---------------------------------------------------
+    def _build_opt_goal_dict(self):#---------------------------------------------------
         self._optimization_goals_dict = {}
         for goal_idx in range(self.awrde.Project.OptGoals.Count):
             self._optimization_goals_dict[goal_idx] = _Goal(self.awrde, goal_idx, 'Optimize')
@@ -1155,6 +1155,361 @@ class _Optimization():
 #
 #*****************************************************************************
 #
+class _Yield():
+    '''
+    methods for using Yiled Analysis
+    '''
+    def __init__(self, awrde):#------------------------------------------
+        self.awrde = awrde
+        #
+    @property
+    def yield_variables_dict(self) -> dict:#-------------------------------
+        '''
+        Return dictionary of variables that are set to use statistics
+        
+        Returns
+        -------
+        yield_variables_dict: dictionary
+            Keys: index
+            Values: Dictionary of fields:
+                       Document Name
+                       Document Type: Schematic, System Diagram, EM Structure, Global Definition
+                       Element Type: Element or Equation
+                       Element Name
+                       Variable Name
+                       Optimize Yield : Boolean
+                       Tolerance in Pct : Boolean
+                       Statistical Variation : float
+                       Statistical Variation 2 : float
+                       Distribution : string
+                       
+        
+        '''
+        yield_variables_dict = {}
+        yield_dict_idx = 0
+        Documents_dict = {0:self.awrde.Project.Schematics,
+                          1:self.awrde.Project.SystemDiagrams,
+                          2:self.awrde.Project.GlobalDefinitionDocuments,
+                          3:self.awrde.Project.EMStructures}
+        DocType_dict = {0:'Circuit Schematic', 1:'System Diagram', 2:'Global Definitions',
+                        3:'EM Structure'}
+        NumDocuments = len(Documents_dict)
+        #
+        for doc_idx in range(NumDocuments):
+            Documents = Documents_dict[doc_idx]
+            for document in Documents:
+                if DocType_dict[doc_idx] == 'Global Definitions':
+                    Elements_object = document.DataElements
+                    Equations_object = document.Equations
+                elif DocType_dict[doc_idx] == 'EM Structure':
+                    Elements_object = document.Schematic.Elements
+                    Equations_object = document.Schematic.Equations
+                else:
+                    Elements_object = document.Elements
+                    Equations_object = document.Equations
+                #end if
+                #
+                for element in Elements_object:
+                    if element.Enabled:
+                        for param in element.Parameters:
+                            if param.UseStatistics:
+                                Fields_dict = {}
+                                Fields_dict['Document Name'] = document.Name
+                                Fields_dict['Document Type'] = DocType_dict[doc_idx]
+                                Fields_dict['Element Type'] = 'Element'
+                                Fields_dict['Variable Name'] = element.Name + ' : '+param.Name
+                                Fields_dict['Yield Optimize'] = param.OptimizeYield
+                                Fields_dict['Tolerance in Pct'] = param.TolInPercent
+                                Fields_dict['Statistical Variation'] = param.StatVariation
+                                Fields_dict['Statistical Variation 2'] = param.StatVariation2
+                                Fields_dict['Distribution'] = self._get_yield_distribution(param.Distribution)
+                                yield_variables_dict[yield_dict_idx] = Fields_dict
+                                yield_dict_idx += 1
+                            #end if
+                        #end for
+                    #end if
+                #end for
+                #
+                for eqn in Equations_object:
+                    if eqn.Enabled:
+                        if eqn.UseStatistics:
+                            Fields_dict = {}
+                            Fields_dict['Document Name'] = document.Name
+                            Fields_dict['Document Type'] = DocType_dict[doc_idx]
+                            Fields_dict['Element Type'] = 'Equation'
+                            Expression = eqn.Expression
+                            Expression = Expression.lstrip()
+                            Expression = Expression.rstrip()
+                            Fields_dict['Variable Name'] = Expression
+                            Fields_dict['Yield Optimize'] = eqn.OptimizeYield
+                            Fields_dict['Tolerance in Pct'] = eqn.TolInPercent
+                            Fields_dict['Statistical Variation'] = eqn.StatVariation
+                            Fields_dict['Statistical Variation 2'] = eqn.StatVariation2
+                            Fields_dict['Distribution'] = self._get_yield_distribution(eqn.Distribution)
+                            yield_variables_dict[yield_dict_idx] = Fields_dict
+                            yield_dict_idx += 1                            
+                        #end if
+                    #end if
+                #end for
+            #end for
+        #end for
+        return yield_variables_dict
+        #
+    def print_yield_variables(self, yield_variables_dict:dict):#-------------------
+        NumYieldVars = len(yield_variables_dict)
+        yv = yield_variables_dict
+        for yield_idx in range(NumYieldVars):
+            print('{:s}{:3.0f}'.format('Yield Variable Index =', yield_idx))
+            print('    {:s} {:s}'.format('Document Name =', yv[yield_idx]['Document Name']))
+            print('    {:s} {:s}'.format('Document Type =', yv[yield_idx]['Document Type']))
+            print('    {:s} {:s}'.format('Element Type =', yv[yield_idx]['Element Type']))
+            if yv[yield_idx]['Element Type'] == 'Element':
+                print('    {:s} {:s}'.format('Element Name : Parameter Name =', yv[yield_idx]['Variable Name']))
+            elif yv[yield_idx]['Element Type'] == 'Equation':
+                print('    {:s}{:s}{:s}'.format('Expression = "', yv[yield_idx]['Variable Name'],'"'))
+            #end if
+            print('    {:s} {:s}'.format('Yield Optimize =',str(yv[yield_idx]['Yield Optimize'])))
+            print('    {:s} {:s}'.format('Tolerance in Pct =',str(yv[yield_idx]['Tolerance in Pct'])))
+            print('    {:s} {:s}'.format('Statistical Variation =',str(yv[yield_idx]['Statistical Variation'])))
+            print('    {:s} {:s}'.format('Statistical Variation 2 =',str(yv[yield_idx]['Statistical Variation 2'])))
+            print('    {:s} {:s}'.format('Distribution =',yv[yield_idx]['Distribution']))
+        #end for
+        #
+    @property
+    def yield_goals_dict(self) -> dict:#-------------------------------------------
+        '''
+        returns dictionary of yield optimizer goals
+        
+        Returns
+        -------
+        optimization_goals_dict: dictionary
+             Keys: Index
+             Values: Dictionary of fields
+        '''
+        self._build_yield_goal_dict()
+        return self._yield_goals_dict
+        #
+    def add_yield_goal(self, measurement_name:str, goal_type:str, goal_xrange:tuple,
+                              goal_yrange:tuple):
+        '''
+        add yield  goal to project
+
+        Parameters
+        ----------
+           measurement_name : string   The measurement name in form of doc name : measurmeent
+           goal_type : string  '>', '<', or '='
+           Xrage : tuple  (xStart, xStop)  must be in base units
+           Yrange : tuple (yStart, yStop)  
+
+        '''
+        if not isinstance(measurement_name, str):
+            raise TypeError('measurement_name must be a string')
+        if not isinstance(goal_type, str):
+            raise TypeError('goal_type must be a string')
+        if not isinstance(goal_yrange, tuple):
+            raise TypeError('goal_xrange must be a tuple')
+        if not isinstance(goal_yrange,tuple):
+            raise TypeError('goal_yrange must be a tuple')        
+        #end if
+        #
+        if goal_type not in {'<','>','='}:
+            raise RuntimeError('invalid goal_type: ' + goal_type)
+        #end if
+        #
+        meas_name_split = measurement_name.split(':')
+        CircuitName = meas_name_split[0]
+        MeasName = meas_name_split[1]
+        if goal_type == '=':
+            GoalTypeEnum = 0
+        elif goal_type == '<':
+            GoalTypeEnum = 1
+        elif goal_type == '>':
+            GoalTypeEnum = 2
+        else:
+            raise RuntimeError('invalid goal_type: ' + goal_type)
+        #end if
+        xStart = goal_xrange[0]
+        xStop = goal_xrange[1]
+        yStart = goal_yrange[0]
+        yStop = goal_yrange[1]
+        xUnit = 0
+        yUnit = 0
+        #
+        try:
+            goal_idx = self.awrde.Project.YieldGoals.Count
+            self.awrde.Project.YieldGoals.Add(CircuitName, MeasName, GoalTypeEnum,
+                                            xStart, xStop, xUnit, yStart, yStop, yUnit)            
+            self._yield_goals_dict[goal_idx] = _Goal(self.awrde, goal_idx, 'Yield')
+        except:
+            warnings.warn('yield goal could not be added')
+        #end try
+        #
+    def remove_yield_goal(self, yield_goal_name:str):#------------------------------
+        '''
+        Remove yield goal
+        '''
+        if not isinstance(yield_goal_name, str):
+            raise TypeError('yield_goal_name must be string type')
+        #end if
+        yield_goal_removed = False
+        #
+        for i in range(len(self._yield_goals_dict)):
+            temp_yield_goal = self._yield_goals_dict[i]
+            if yield_goal_name == temp_yield_goal.goal_name:
+                dict_idx = i
+                break
+            #end if
+        #end for
+        #
+        for yield_goal_idx in range(1, self.awrde.Project.YieldGoals.Count+1):
+            temp_yield_goal_name = self.awrde.Project.YieldGoals(yield_goal_idx).Name
+            if temp_yield_goal_name == yield_goal_name:
+                self.awrde.Project.YieldGoals.Remove(yield_goal_idx)
+                yield_goal_removed = True
+                self._yield_goals_dict.pop(dict_idx)
+                break
+            #end if
+        #end for
+        if not yield_goal_removed:
+            warnings.warn('Yield Goal ' + yield_goal_name + ' did not get removed')
+        #end for
+        return yield_goal_removed
+        #
+    @property
+    def yield_type_list(self) -> list:#-------------------------------------------
+        ''' returns list of the yield methods'''
+        yield_type_list = list()
+        for i in range(1, self.awrde.Project.YieldAnalyzer.TypeCount+1):
+            yield_type_list.append(self.awrde.Project.YieldAnalyzer.TypeName(i))
+        #end for
+        return yield_type_list
+        #
+    @property
+    def yield_type(self) -> str: #-----------------------------------------------
+        '''
+        returns yield analyzer type name
+        '''
+        TypeNum = self.awrde.Project.YieldAnalyzer.Type
+        self._build_yield_properties_dict()  #build the properties dictionary upon querry of the optimization type
+        return self.awrde.Project.YieldAnalyzer.TypeName(TypeNum)
+        #
+    @yield_type.setter
+    def yield_type(self, yield_type_name: str):#---------------------------
+        '''sets the current yield analyzer type'''
+        if not isinstance(yield_type_name, str):
+            raise TypeError('yield_type_name must be a string')
+        #end if
+        found_it = False
+        for i in range(1, self.awrde.Project.YieldAnalyzer.TypeCount+1):
+            if yield_type_name == self.awrde.Project.YieldAnalyzer.TypeName(i):
+                found_it = True
+                break
+            #end if
+        #end for
+        if not found_it:
+            raise RuntimeError('Invalid yield type: ' + yield_type_name)
+        #end if
+        self.awrde.Project.YieldAnalyzer.Type = i
+        self._build_yield_properties_dict() #Update the optimization properties dictionary upon changing the optimization type
+        #
+    @property
+    def yield_type_properties(self) -> dict:#---------------------------------------------
+        '''
+        returns a dictionary of properties for the current yield analyzer type
+        
+        Returns
+        -------
+        yield_properties_dict: dictionary
+                             keys are the property name
+                             values are the property values
+        '''
+        try:
+            yield_properties_dict = self._yield_properties_dict
+        except:
+            self._build_yield_properties_dict()
+            yield_properties_dict = self._yield_properties_dict
+        #end try
+        return yield_properties_dict
+        #
+    def yield_update_type_properties(self):#----------------------------------------------
+        '''update the yield type properties'''
+        try:
+            for prop in self.awrde.Project.YieldAnalyzer.Properties:
+                prop.Value = self._yield_properties_dict[prop.Name]
+            #end for
+        except:
+            warnings.warn('Yield properties not updated')
+        #end try
+        #
+    @property
+    def yield_max_iterations(self) -> float:#------------------------------------------
+        ''' returns the Optimizer MaxIterations'''
+        return self.awrde.Project.YieldAnalyzer.MaxIterations
+        #
+    @yield_max_iterations.setter
+    def yield_max_iterations(self, max_iterations: float):#---------------------------
+        '''sets the Yield Analyzer MaxIterations'''
+        if (not isinstance(max_iterations, float)) and (not isinstance(max_iterations, int)):
+            raise TypeError('max_iterations must be either a float or int type')
+        #end if
+        #
+        try:
+            self.awrde.Project.YieldAnalyzer.MaxIterations = max_iterations
+        except:
+            warnings.warn('Yield Analyzer max iterations could not be set')
+        #end try
+        self.awrde.Project.YieldAnalyzer.MaxIterations = int(max_iterations)
+        #
+    @property
+    def yield_analysis_start(self) -> bool:#---------------------------------------------------------
+        return self.awrde.Project.YieldAnalyzer.Running
+        #
+    @yield_analysis_start.setter
+    def yield_analysis_start(self, yield_start: bool):#----------------------------------------
+        if not isinstance(yield_start, bool):
+            raise TypeError('yield_start must be boolean type')
+        #end if
+        if yield_start:
+            self.awrde.Project.YieldAnalyzer.start()  #yes, lower case start()
+        else:
+            self.awrde.Project.YieldAnalyzer.stop()  #yes, lower case stop()
+        #end if
+        #    
+    def _get_yield_distribution(self, param_distribution):#-------------------------
+        if param_distribution == 1:
+            distribution_str = 'Uniform'
+        elif param_distribution == 2:
+            distribution_str = 'Normal'
+        elif param_distribution == 4:
+            distribution_str = 'Log-Normal'
+        elif param_distribution == 5:
+            distribution_str = 'Discrete'
+        elif param_distribution == 6:
+            distribution_str = 'Normal Minus Tol'
+        elif param_distribution == 7:
+            distribution_str = 'Normal Clipped' 
+        else:
+            distribution_str = 'Not Recognized'
+        #end if
+        return distribution_str
+        #
+    def _build_yield_goal_dict(self):#----------------------------------------------
+        self._yield_goals_dict = {}
+        for goal_idx in range(self.awrde.Project.YieldGoals.Count):
+            print('hi')
+            self._yield_goals_dict[goal_idx] = _Goal(self.awrde, goal_idx, 'Yield')
+        #end for
+        #
+    def _build_yield_properties_dict(self) -> dict:#--------------------------------------------
+        ''' builds a dictionary of yield properties '''
+        self._yield_properties_dict = {}
+        for prop in self.awrde.Project.YieldAnalyzer.Properties:
+            self._yield_properties_dict[prop.Name] = prop.Value
+        #end for
+        #    
+#
+#*****************************************************************************
+#
 class _GlobalDefinitions():
     '''
     Methods for collection of Global Definitions Documents
